node-line-readable-stream
==============


This is a port of [bluesmoon's node-line.input-stream](https://github.com/bluesmoon/node-line-input-stream). Provides a stream wrapper for a node.js Readable Stream for line-by-line reading.

How to install
---------------

Node:

	npm install line-readable-stream
	
Example Usage
----------------

Like [Readable Stream](http://nodejs.org/api/stream.html#stream_readable_stream) with a <code>line</code> event.

JavaScript:
```JavaScript
var LineReadableStream = require('line-readable-stream');
var fs = require('fs');

var stream = new LineReadableStream(fs.createReadStream("foo.txt", { flags: "r" }));
stream.setEncoding("utf8");
stream.setDelimiter("\n");  // optional string, defaults to "\n"

stream.on("error", function(err) {
		console.log(err);
	});

stream.on("data", function(chunk) {
		// You don't need to use this event
	});

stream.on("line", function(line) {
		// Sends you lines from the stream delimited by delimiter
	});

stream.on("end", function() {
		// No more data, all line events emitted before this event
	});

stream.on("close", function() {
		// Same as ReadableStream's close event
	});

if(stream.readable) {
	console.log("stream is readable");
}

// Also available: resume(), pipe()
```

TypeScript:

```TypeScript
///<require path="typings/node/node.d.ts"/>
///<require path="node_modules/line-readable-stream/LineReadableStream.d.ts"/>

// If you know a better war to include the .ts file that comes with the npm package, let me know.
import LineReadableStream = require('node_modules/line-readable-stream/LineReadableStream');
import fs = require('fs');

var stream = new LineReadableStream(fs.createReadStream("foo.txt", { flags: "r" }));
stream.setEncoding("utf8");
stream.setDelimiter("\n");  // optional string, defaults to "\n"

stream.on("error", err => console.log(err));

stream.on("data", chunk => {
		// You don't need to use this event
	});

stream.on("line", line => {
		// Sends you lines from the stream delimited by delimiter
		console.log("Line: " + line);
	});

stream.on("end", () => {
		// No more data, all line events emitted before this event
	});

stream.on("close", () => {
		// Same as ReadableStream's close event
	});

if(stream.readable)
	console.log("stream is readable");


// Also available: resume(), pipe()
```

Usage information
-----------------


You can also attach listeners to any event specific to the underlying stream, ie,
you can listen to the <code>open</code> event for streams created by [<code>fs.createReadStream()</code>](http://nodejs.org/api/fs.html#fs_fs_createreadstream_path_options)
or the <code>connect</code> event for [Net](http://nodejs.org/api/net.html) streams.

A side effect of this is that you can add a listener for any junk string and <code>LineReadableStream</code> will
pretend that it worked.  The event listener may never be called though.


- The delimiter is not included in the line passed to the <code>line</code> handler
- Even though this is called <code>line-readable-stream</code>, you can delimit by any string, so for example, setting delimiter to `"\n\n"` will read by paragraph (sort of).
- You can set the delimiter to a regular expression, which let's you do cool things like drop multiple blank lines: <code>/[\r\n]+/</code>
- All methods return `this`, so can be chained
